/*
 * Copyright (c) 2018. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.service.jpa;

import javax.persistence.EntityManager;

import org.hedgecode.chess.domain.DomainObject;
import org.hedgecode.chess.persistence.PersistenceManager;
import org.hedgecode.chess.service.DomainService;
import org.hedgecode.chess.service.ServiceRegistry;

/**
 *
 *
 * @author Dmitry Samoshin aka gotty
 */
public abstract class JpaDomainService<DomainType extends DomainObject> implements DomainService<DomainType> {

    JpaDomainService() {
    }

    EntityManager getEntityManager() {
        PersistenceManager persistenceManager = ServiceRegistry.singleProvider(
                PersistenceManager.class
        );
        return persistenceManager.getEntityManager();
    }

    @Override
    public DomainType create(DomainType domainObject) {
        getEntityManager().persist(
                domainObject
        );
        return domainObject;
    }

    @Override
    public DomainType update(DomainType domainObject) {
        getEntityManager().persist(
                domainObject
        );
        return domainObject;
    }

    @Override
    public void remove(DomainType domainObject) {
        getEntityManager().remove(
                domainObject
        );
    }

}
