/*
 * Copyright (c) 2018-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.pgn;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * PGNUtils
 *
 * @author Dmitry Samoshin aka gotty
 */
public final class PGNUtils {

    private static final char BACKSLASH = '\\';
    private static final String SHIELD_REGEX = "\\\\";

    private static final String CRLF = "\\r?\\n";
    private static final String SPACE = " ";

    public static String match(String source, String regex) {
        Matcher matcher = Pattern.compile(
                regex,
                Pattern.MULTILINE
        ).matcher(source);
        if (matcher.find()) {
            return matcher.groupCount() > 0
                    ? matcher.group(1)
                    : matcher.group();
        }
        return null;
    }

    public static boolean isPgn(String source) {
        return match(
                source,
                PGNConstants.PGN_DETECT_REGEX
        ) != null;
    }

    public static String shield(String source, char[] shields) {
        for (char shield : shields) {
            if (source.indexOf(shield) >= 0) {
                String regexShield =
                        shield == BACKSLASH
                                ? SHIELD_REGEX
                                : String.valueOf(shield);
                source = source.replaceAll(
                        String.format("([%s])", regexShield),
                        SHIELD_REGEX.concat("$1")
                );
            }
        }
        return source;
    }

    public static String stripCrlf(String pgn) {
        return pgn.replaceAll(CRLF, SPACE);
    }

    private PGNUtils() {
        throw new AssertionError(
                String.format("No %s instances!", getClass().getName())
        );
    }

}
