/*
 * Copyright (c) 2018-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.pgn.entity;

import java.util.ArrayList;
import java.util.List;

/**
 * DetailMove
 *
 * @author Dmitry Samoshin aka gotty
 */
public class DetailMove implements Move {

    private int ply;
    private String move;

    private List<String> comments = new ArrayList<>();
    private List<Variation> variations = new ArrayList<>();

    public DetailMove(int ply, String move) {
        this.ply = ply;
        this.move = move;
    }

    @Override
    public int ply() {
        return ply;
    }

    @Override
    public String move() {
        return move;
    }

    public void addComment(String comment) {
        comments.add(comment);
    }

    public void addVariation(Variation variation) {
        variations.add(variation);
    }

    public List<String> comments() {
        return comments;
    }

    public List<Variation> variations() {
        return variations;
    }

}
