/*
 * Copyright (c) 2018-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.pgn.format;

import java.util.Collections;
import java.util.List;

import org.hedgecode.chess.pgn.entity.Move;
import org.hedgecode.chess.pgn.entity.Moves;

import static org.hedgecode.chess.pgn.PGNConstants.*;

/**
 * ReduceMovesFormat
 *
 * @author Dmitry Samoshin aka gotty
 */
public class ReduceMovesFormat extends AbstractMovesFormat {

    @Override
    public String format(List<Move> moves) {
        Collections.sort(moves);
        StringBuilder sb = new StringBuilder();
        for (Move move : moves) {
            sb.append(
                    formatSan(move)
            ).append(
                    PGN_SPACE
            );
        }
        return sb.toString().trim();
    }

    @Override
    public String format(Moves moves) {
        return format(
                moves.getMoves().toArray(
                        new Move[moves.getMoves().size()]
                )
        );
    }

    private String formatSan(Move move) {
        return move.ply() % 2 != 0
                ? String.format(WHITE_MOVE_FORMAT, move.ply() / 2 + 1, move.move())
                : String.format(BLACK_MOVE_FORMAT, move.move());
    }

}
