/*
 * Copyright (c) 2018-2020. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.pgn.token;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.hedgecode.chess.pgn.PGNConstants;
import org.hedgecode.chess.pgn.entity.Game;
import org.hedgecode.chess.position.ParseException;

/**
 * TagToken
 *
 * @author Dmitry Samoshin aka gotty
 */
public class TagToken implements Token<Game> {

    private static final String TAG_REGEX = "^\\[([^\"]+)\\s\"([^\"]+)\"\\]$";
    private static final Pattern TAG_PATTERN = Pattern.compile(TAG_REGEX, Pattern.MULTILINE);
    private static final int NAME_GROUP = 1, VALUE_GROUP = 2;

    @Override
    public int token(Game game, String pgn) throws ParseException {
        Matcher matcher = TAG_PATTERN.matcher(pgn);
        if (!matcher.find()) {
            throw new ParseException("parse.pgn.incorrect.tag", pgn);
        } else {
            String name = matcher.group(NAME_GROUP);
            String value = matcher.group(VALUE_GROUP);
            game.addTag(name, value);
        }
        return pgn.indexOf(PGNConstants.PGN_CRLF) + 1;
    }

}
