/*
 * Copyright (c) 2018-2019. Developed by Hedgecode.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.hedgecode.chess.img.bitmap;

import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.imageio.ImageIO;

import org.hedgecode.chess.img.ImageException;
import org.hedgecode.chess.img.ImageShaper;

/**
 * Shaper for bitmap images.
 *
 * @author Dmitry Samoshin aka gotty
 */
public class BitmapImageShaper implements ImageShaper {

    private BitmapImageShaper() {
    }

    @Override
    public BufferedImage shape(InputStream imageStream, int size) throws ImageException {
        try {
            return scaledImage(
                    ImageIO.read(imageStream),
                    size
            );
        } catch (IOException e) {
            throw new ImageException("image.unable.load.bitmap", e.getMessage());
        }
    }

    @Override
    public BufferedImage shape(URL imageUrl, int size) throws ImageException {
        try {
            return scaledImage(
                    ImageIO.read(imageUrl),
                    size
            );
        } catch (IOException e) {
            throw new ImageException("image.unable.load.bitmap", e.getMessage());
        }
    }

    @Override
    public BufferedImage shape(File imageFile, int size) throws ImageException {
        try {
            return scaledImage(
                    ImageIO.read(imageFile),
                    size
            );
        } catch (IOException e) {
            throw new ImageException("image.unable.load.bitmap", e.getMessage());
        }
    }

    private BufferedImage scaledImage(BufferedImage image, int size) {
        if (image.getWidth() == size && image.getHeight() == size) {
            return image;
        }
        BufferedImage scaledImage = new BufferedImage(size, size, BufferedImage.TYPE_INT_ARGB);
        Graphics graphics = scaledImage.createGraphics();
        graphics.drawImage(
                image.getScaledInstance(size, size, BufferedImage.SCALE_SMOOTH),
                0, 0, null
        );
        graphics.dispose();
        return scaledImage;
    }

    public static ImageShaper create() {
        return new BitmapImageShaper();
    }

}
